/* Weather.c >>>>>>>>>>>>>>>>>>>    */

#include <io.h>
#include <stdlib.h>
#include <sig-avr.h> 
#include <interrupt.h> 
#include <string-avr.h>
#include <ctype.h>
#include <progmem.h>

#include "StdDefs.h"
#include "mbRFslv.h" 

#define mbADDR 30
#define GO_TO_SLEEP()   asm volatile ("sleep"::)
#define SENSOR_POWER_ON()  sbi(PORTD, 6)
#define SENSOR_POWER_OFF() cbi(PORTD, 6)

INT16U LC;
CHARU Temp1, Temp2, Temp3, Temp4, Light, Volts;
CHARU cSend;

void initialize(void);
void SetSendRate(void);
void GetAndSend(void);
INT16U read_analog(CHARU mux);
CHARU  linearize_temp(int raw);
CHARU  linearize_light(int raw);
CHARU linearize_volts(int raw);
CHARU subt_rollover(CHARU x, CHARU y);
void SendMaster(void);
void print_info(int);

void LEDsOff(int x);
CHARU DataBuf[8];   // Data buffer for incoming, outgoing data
CHARU *BufPtr2;		 // Pointer to DataBuf

prog_char Thermistor[2][10] = 
     //  Temperature in F * 10, Raw readings from ADC
     {{172,150,128,115,103, 86, 74, 56, 47, 39},
      { 61, 70, 78, 85, 99,124,142,167,183,194}};              

int main (void)
   {
	initialize();
	for (;;)
		{
       MCUCR = 0x20;      // Enable sleep, idle mode
       SENSOR_POWER_OFF();
       GO_TO_SLEEP();
       //print_info(25);
       msleep(20);
       LC++;
		}   
   return 0;       
}

void initialize(void)
	{
   //CHARU c;
	cli();

   cbi(UCSRB,4);       // Ensure UART receive is disabled
   
   outp(0xFD, PORTB);  // Unused inputs pulled high
   outp(0x02, DDRB);   // PB1 OCR1
   outp(0x00, PORTC);   // All analog inputs
   outp(0x00, DDRC);
   outp(0x0D, PORTD);  // See notes     
   outb(0xE2, DDRD);
   
   TCCR1A = 0x40;      // TCNT1 enabled, toggle OC1, no PWM
   TCCR1B = 0x1D;      // Clear TCNT1 on output compare match, divide by 1024


   TCCR0 = 0x07;       // TCNT0 enabled, external, rising edge
   TIMSK = 0x02;       // TCNT0 overflow interrupt enabled
   

   
   InitializeRF();
   cbi(UART_CONTROL_REG,4);       // Ensure UART receive is disabled
   BufPtr2 = &DataBuf[0];
   SetSendRate();

   sei();
   GetAndSend();
   GetAndSend();
   }
   
SIGNAL (SIG_OVERFLOW0)
   {
   GetAndSend();
   }
   
void SetSendRate(void)
   {
   // Determine send rate from dip switch settings
   cSend = 0;
   if(bit_is_clear(PINB,0)) cSend = 0x01;
   if(bit_is_clear(PINB,2)) cSend |= 0x02;
   if(bit_is_clear(PIND,0)) cSend |= 0x04;
   if(bit_is_clear(PIND,3)) cSend |= 0x08;
   
   switch(cSend)
       {
       case 0:  OCR1 = 7;      break;  // 1 second
       case 1:  OCR1 = 15;     break;  // 2 second
       case 2:  OCR1 = 38;     break;  // 5 second
       case 3:  OCR1 = 76;     break;  // 10 second
       case 4:  OCR1 = 229;    break;  // 30 second
       case 5:  OCR1 = 458;    break;  // 1 minute
       case 6:  OCR1 = 916;    break;  // 2 minute
       case 7:  OCR1 = 2289;   break;  // 5 minute
       case 8:  OCR1 = 4578;   break;  // 10 minute
       case 9:  OCR1 = 6867;   break;  // 15 minute
       case 10: OCR1 = 13733;  break;  // 30 minute
       case 11: OCR1 = 27466;  break;  // 60 minute
       case 12: OCR1 = 41199;  break;  // 90 minute
       case 13: OCR1 = 54932;  break;  // 2 hour
       }   
   }   
   
void GetAndSend(void)
   {
   SENSOR_POWER_ON();
   RUNLED_ON();
   ADCSR = 0x84;       // Enables A/D, resets done flag
   msleep(1000);
   Temp1 = linearize_temp(read_analog(0));
   Temp2 = linearize_temp(read_analog(1));
   Temp3 = linearize_temp(read_analog(2));
   Temp4 = linearize_temp(read_analog(3));
   Light = linearize_light(read_analog(4));
   Volts = linearize_volts(read_analog(5));
   SENSOR_POWER_OFF();
   RUNLED_OFF();
   ADCSR = 0;          // Disable A/D
   SendMaster();   
   }      

// Fixes the main loop duration at x 1/10ths of ms
// Each TCNT0 count is 0.256 ms
   
// Subtract y from x - handles timer rollover
CHARU subt_rollover(CHARU x, CHARU y)
   {
   if (x >= y)
       return x - y;
   else
       return (256 - y) + x;
   }   

// comm LEDs off every x calls to LEDsOff    
void LEDsOff(int x)  
   {
	static int c;
	if (c > x)
	   {
      RF_LED_OFF();
      c = 0;
	   }
	else
	   c++;   
   } 
   
// Reads a single analog pin, mux = 0 thru 7 same as pin
INT16U read_analog(CHARU mux)
   {
   ADMUX = mux;
   sbi(ADCSR, 6);     // Start conversion
   while(bit_is_clear(ADCSR, 4));     // Wait until conversion is complete
   sbi(ADCSR, 4);    // Clear flag
	return (INT16U)(ADCL | ((int)ADCH<<8));
   }
 
// 10K nominal Thermistor linearize routine - straight line interpolation
// In order to force the linearization table into flash and not RAM
// must use the following method:
// #include <progmem.h>
// Linearization table stored as 
//          prog_char Thermistor[2][10] =
// Read values with PRG_RDB(&Thermistor[x][y]) - see below
//  
CHARU linearize_temp(int raw)
	 {
	 CHARU j;
    CHARU lin;
    int rawh, rawl, linh, linl;
    raw = raw>>2;
	 for (j = 8; j > -1; j--)
       {
    	rawl = PRG_RDB(&Thermistor[1][j]);
       rawh = PRG_RDB(&Thermistor[1][j+1]);
       linl = PRG_RDB(&Thermistor[0][j]);
       linh = PRG_RDB(&Thermistor[0][j+1]);
		
		if (raw > rawl)
			{
			lin = linl - (((linl - linh)*
				(raw - rawl) /
				(rawh - rawl)));
				break;
				}
           else
              lin = 255;
        }

	 return (CHARU)lin;

	 }

CHARU linearize_light(int raw)
	 {
    return 255 - (CHARU)(raw >> 2);
	 }
    
CHARU linearize_volts(int raw)
   {
   return (CHARU)(raw / 5);
   }    
   
void SendMaster(void)
   {
   DataBuf[0] = Temp1;
   DataBuf[1] = Temp2;
   DataBuf[2] = Temp3;
   DataBuf[3] = cSend;   // Temporary - testing
   DataBuf[4] = Light;
   DataBuf[5] = Volts;
       
   SendToMaster(mbADDR, 6, BufPtr2);
   }      
	
void print_info(int x)
   {
   static int c;
   if (c >= x)
	     {  
        putchar('A'); putBCD(Temp1, 5, 1);
        putchar('B'); putBCD(Temp2, 5, 1);
        putchar('C'); putBCD(Temp3, 5, 1);
        putchar('D'); putBCD(Temp4, 5, 1);
        putchar('L'); putBCD(Light, 5, 1);
        putchar('V'); putBCD(Volts, 5, 1);
        CRLF();
        c = 0;
        }
   else
       c++;
   }
   
